
package logo;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;


/*************************************************************************

	Un petit Logo minimal qui devra tre amlior par la suite

				J. Ferber - 1999-2001

				Cours de DESS TNI - Montpellier II

	@version 2.0
	@date 25/09/2001

**************************************************************************/

/** La classe Tortue qui se dplace en coordonnes polaires
**/

class Tortue
{

	double convDegGrad = 0.0174533; // la constante de conversion de degres en gradient


	JPanel feuille;	// la feuille de dessin sur laquelle dessine la tortue


	int x, y;	// les coordonnees de la tortue
	int dir;	// la direction de la tortue
	boolean crayon=true; // par defaut on suppose qu'on dessine
	int coul;


	public Tortue(JPanel f){
		feuille = f;
		reset();
	}

	void reset()
  	{	// on initialise la position de la tortue
		x = 250;
		y = 180;
		dir = -90;
		crayon = true;
  	}

	protected static final int rp=10,rb=5;
	protected void drawTurtle (Graphics g){
      //Calcule les 3 coins du triangle a partir de
      // la position de la tortue p
	  Point p = new Point(x,y);
	  Polygon arrow = new Polygon();

      //Calcule des deux bases
      //Angle de la droite
      double theta=convDegGrad*(-dir);
      //Demi angle au sommet du triangle
      double alpha=Math.atan( (float)rb / (float)rp );
      //Rayon de la fleche
      double r=Math.sqrt( rp*rp + rb*rb );
      //Sens de la fleche

      //Pointe
	  Point p2=new Point((int) Math.round(p.x+r*Math.cos(theta)),
						 (int) Math.round(p.y-r*Math.sin(theta)));
	  arrow.addPoint(p2.x,p2.y);
	  arrow.addPoint((int) Math.round( p2.x-r*Math.cos(theta + alpha) ),
		  (int) Math.round( p2.y+r*Math.sin(theta + alpha) ));

      //Base2
	  arrow.addPoint((int) Math.round( p2.x-r*Math.cos(theta - alpha) ),
		  (int) Math.round( p2.y+r*Math.sin(theta - alpha) ));

	  arrow.addPoint(p2.x,p2.y);
	  g.setColor(Color.green);
	  g.fillPolygon(arrow);
    }

  	Color getColor(int c) {
  		switch(c) {
  			case 0: return(Color.black);
  			case 1: return(Color.blue);
  			case 2: return(Color.cyan);
  			case 3: return(Color.darkGray);
  			case 4: return(Color.red);
  			case 5: return(Color.green);
  			case 6: return(Color.lightGray);
  			case 7: return(Color.magenta);
  			case 8: return(Color.orange);
  			case 9: return(Color.gray);
  			case 10: return(Color.pink);
  			case 11: return(Color.yellow);
  			default : return(Color.black);
		}
  	}

	/** les procedures de base de fonctionnement de la tortue */

	// avancer de n pas
	void avancer(int dist)
	{	Graphics g = feuille.getGraphics();

		int newX = (int) Math.round(x+dist*Math.cos(convDegGrad*dir));
		int newY = (int) Math.round(y+dist*Math.sin(convDegGrad*dir));
		if (crayon) {
			g.setColor(getColor(coul));
			g.drawLine(x,y,newX,newY);
		}
		x = newX;
		y = newY;
		if (getShowTurtle())
		  this.drawTurtle(g);
		System.out.println("dir: " + dir);
	}

	// aller  droite
	void droite(int ang)
	{
		dir = (dir + ang) % 360;
		if (getShowTurtle())
		  this.drawTurtle(feuille.getGraphics());
		System.out.println("dir: " + dir);
	}

	// aller  gauche
	void gauche(int ang)
	{
		dir = (dir - ang) % 360;
		if (getShowTurtle())
		  this.drawTurtle(feuille.getGraphics());
		System.out.println("dir: " + dir);
	}

	//show turtle
	private boolean showTurtle=true;
	public void setShowTurtle(boolean b){
	  showTurtle=b;
	}
	public boolean getShowTurtle(){
	  return(showTurtle);
	}

	// baisser le crayon pour dessiner
	void baisserCrayon()
	{
		crayon = true;
	}

	// lever le crayon pour ne plus dessiner
	void leverCrayon()
	{
		crayon = false;
	}

	// pour changer de couleur de dessin
	void couleur(int n)
	{
		coul = n % 12;
	}

	void couleurSuivante() {
	 	couleur(coul+1);
	}

	/** quelques classiques */

	void carre(int n) {
		for (int i=0;i<4;i++) {
			avancer(100);
			droite(90);
		}
	}

	void poly(int n, int a) {
		for (int j=0;j<a;j++) {
			avancer(n);
			droite(360/a);
		}
	}

	void spiral(int n, int k, int a) {
		for (int i = 0; i < k; i++) {
			couleur(coul+1);
			avancer(n);
			droite(360/a);
			n = n+1;
		}
	}
}


public class SimpleLogo extends JFrame implements ActionListener {

	JPanel feuille;

	Tortue courante;


	// la procedure principale
	public static void main(String[] args)
	{
		System.out.println( "Logo demarre!" );
		new SimpleLogo();
	}


  	void quitter()
  	{
  		System.exit(0);
  	}


	public SimpleLogo()
	{
		super("un logo tout simple");
		getContentPane().setLayout(new BorderLayout(10,10));

		setDefaultCloseOperation(EXIT_ON_CLOSE);
		// les primitives (les boutons du haut)
		JPanel p1 = new JPanel(new GridLayout());
		JButton b10 = new JButton("Avancer 50");
		p1.add(b10);
		b10.addActionListener(this);
		JButton b11 = new JButton("Droite 45");
		p1.add(b11);
		b11.addActionListener(this);
		JButton b12 = new JButton("Gauche 45");
		p1.add(b12);
		b12.addActionListener(this);

		JButton b15 = new JButton("LeverCrayon");
		p1.add(b15);
		b15.addActionListener(this);
		JButton b16 = new JButton("BaisserCrayon");
		p1.add(b16);
		b16.addActionListener(this);
		JButton b17 = new JButton("Couleur suiv.");
		p1.add(b17);
		b17.addActionListener(this);
		getContentPane().add(p1,"North");

		// les boutons du bas
		JPanel p2 = new JPanel(new GridLayout());
		JButton b20 = new JButton("Proc1");
		p2.add(b20);
		b20.addActionListener(this);
		JButton b21 = new JButton("Proc2");
		p2.add(b21);
		b21.addActionListener(this);
		JButton b22 = new JButton("Proc3");
		p2.add(b22);
		b22.addActionListener(this);

		JButton b23 = new JButton("Effacer");
		p2.add(b23);
		b23.addActionListener(this);
		JButton b24 = new JButton("Quitter");
		p2.add(b24);
		b24.addActionListener(this);
		getContentPane().add(p2,"South");

		feuille = new JPanel();
		feuille.setSize(500,360);
		feuille.setBackground(Color.white);
		feuille.setPreferredSize(new Dimension(500,400));
		getContentPane().add(feuille,"Center");


		// on cre la tortue qui initialise sa position
		courante = new Tortue(feuille);

		pack();
  		show();
	}


	/** la gestion des actions des boutons */
	public void actionPerformed(ActionEvent e)
	{
        String c = e.getActionCommand();

        // actions des boutons du haut
        if (c.equals("Avancer 50")) courante.avancer(50);
        else if (c.equals("Droite 45"))  courante.droite(45);
        else if (c.equals("Gauche 45"))  courante.gauche(45);
        else if (c.equals("LeverCrayon"))  courante.leverCrayon();
        else if (c.equals("BaisserCrayon"))  courante.baisserCrayon();
        else if (c.equals("Couleur suiv.")) {  courante.couleurSuivante();}

        // actions des boutons du bas
        else if (c.equals("Proc1"))  proc1();
        else if (c.equals("Proc2"))  proc2();
        else if (c.equals("Proc3"))  proc3();
        else if (c.equals("Effacer")) effacer();
        else if (c.equals("Quitter")) quitter();
  	}

  	/** les procedures Logo qui combine plusieurs commandes..*/
	void proc1()
	{
		courante.carre(100);
	}

	void proc2()
	{
		courante.poly(60,8);
	}

	void proc3()
	{
	  	courante.spiral(50,40,6);
	}



  	// efface tout et reinitialise la tortue
  	void effacer()
  	{
  		feuille.repaint();
  		courante.reset();
  	}



}
